@@ -3,10 +3,13 @@
 nop.py
 
 Created by Thomas Mangin on 2009-11-06.
-Copyright (c) 2009-2015 Exa Networks. All rights reserved.
+Copyright (c) 2009-2017 Exa Networks. All rights reserved.
+License: 3-clause BSD. (See the COPYRIGHT file)
 """
 
-from exabgp.bgp.message import Message
+from exabgp.util import character
+
+from exabgp.bgp.message.message import Message
 
 # ========================================================================= NOP
 #
@@ -14,10 +17,10 @@ from exabgp.bgp.message import Message
 
 class NOP (Message):
 	ID = Message.CODE.NOP
-	TYPE = chr(Message.CODE.NOP)
+	TYPE = character(Message.CODE.NOP)
 
-	def message (self):
-		return self._message(self.data)
+	def message (self,negotiated=None):
+		raise RuntimeError('NOP messages can not be sent on the wire')
 
 	def __str__ (self):
 		return "NOP"
