@@ -1,10 +1,11 @@
 # encoding: utf-8
 """
-encapsulation.py
+l2info.py
 
 Created by Thomas Mangin on 2014-06-20.
-Copyright (c) 2014-2015 Orange. All rights reserved.
-Copyright (c) 2014-2015 Exa Networks. All rights reserved.
+Copyright (c) 2014-2017 Orange. All rights reserved.
+Copyright (c) 2014-2017 Exa Networks. All rights reserved.
+License: 3-clause BSD. (See the COPYRIGHT file)
 """
 
 from struct import pack
@@ -16,6 +17,7 @@ from exabgp.bgp.message.update.attribute.community.extended import ExtendedCommu
 # RFC 4761
 
 
+@ExtendedCommunity.register
 class L2Info (ExtendedCommunity):
 	COMMUNITY_TYPE = 0x80
 	COMMUNITY_SUBTYPE = 0x0A
@@ -32,13 +34,13 @@ class L2Info (ExtendedCommunity):
 			self,
 			community if community is not None else pack(
 				"!2sBBHH",
-				self._packedTypeSubtype(),
+				self._subtype(),
 				encaps,control,
 				mtu,reserved
 			)
 		)
 
-	def __str__ (self):
+	def __repr__ (self):
 		return "l2info:%s:%s:%s:%s" % (self.encaps,self.control,self.mtu,self.reserved)
 
 	@staticmethod
