@@ -3,7 +3,8 @@
 environment.py
 
 Created by Thomas Mangin on 2011-11-29.
-Copyright (c) 2011-2015 Exa Networks. All rights reserved.
+Copyright (c) 2011-2017 Exa Networks. All rights reserved.
+License: 3-clause BSD. (See the COPYRIGHT file)
 """
 
 # XXX: raised exception not caught
@@ -16,6 +17,7 @@ import sys
 import pwd
 import syslog
 
+from exabgp.protocol.ip import IP
 from exabgp.util.ip import isip
 
 
@@ -25,6 +27,8 @@ from exabgp.util.ip import isip
 class NoneDict (dict):
 	def __getitem__ (self, name):
 		return None
+
+
 nonedict = NoneDict()
 
 
@@ -99,6 +103,10 @@ class environment (object):
 	def quote (_):
 		return "'%s'" % str(_)
 
+	@staticmethod
+	def quote_list (_):
+		return "'%s'" % ' '.join([str(x) for x in _])
+
 	@staticmethod
 	def nop (_):
 		return _
@@ -133,10 +141,16 @@ class environment (object):
 		raise TypeError('ip %s is invalid' % _)
 
 	@staticmethod
-	def optional_ip (_):
-		if not _ or isip(_):
-			return _
-		raise TypeError('ip %s is invalid' % _)
+	def ip_list (_):
+		ips = []
+		for ip in _.split(' '):
+			if not ip:
+				continue
+			elif isip(ip):
+				ips.append(IP.create(ip))
+			else:
+				raise TypeError('ip %s is invalid' % ip)
+		return ips
 
 	@staticmethod
 	def user (_):
@@ -221,6 +235,14 @@ class environment (object):
 				return name
 		raise TypeError('invalid log level %s' % log)
 
+	@staticmethod
+	def umask_read (_):
+		return int(_, 8)
+
+	@staticmethod
+	def umask_write (_):
+		return "'%s'" % (oct(_))
+
 	@staticmethod
 	def default ():
 		for section in sorted(environment.configuration):
@@ -259,84 +281,16 @@ class environment (object):
 					continue
 				yield "%s.%s.%s=%s" % (environment.application,section,k,environment.configuration[section][k]['write'](v))
 
-	# Compatibility with 2.0.x
-	@staticmethod
-	def _compatibility (env):
-		profile = os.environ.get('PROFILE','')
-		if profile:
-			env.profile.enable = True
-		if profile and profile.lower() not in ['1','true','yes','on','enable']:
-			env.profile.file = profile
-
-		# PDB : still compatible as a side effect of the code structure
-
-		syslog_env = os.environ.get('SYSLOG','')
-		if syslog_env != '':
-			env.log.destination = syslog_env
-
-		if os.environ.get('DEBUG_SUPERVISOR','').lower() in ['1','yes']:
-			env.log.reactor = True
-		if os.environ.get('DEBUG_DAEMON','').lower() in ['1','yes']:
-			env.log.daemon = True
-		if os.environ.get('DEBUG_PROCESSES','').lower() in ['1','yes']:
-			env.log.processes = True
-		if os.environ.get('DEBUG_CONFIGURATION','').lower() in ['1','yes']:
-			env.log.configuration = True
-		if os.environ.get('DEBUG_WIRE','').lower() in ['1','yes']:
-			env.log.network = True
-			env.log.packets = True
-		if os.environ.get('DEBUG_MESSAGE','').lower() in ['1','yes']:
-			env.log.message = True
-		if os.environ.get('DEBUG_RIB','').lower() in ['1','yes']:
-			env.log.rib = True
-		if os.environ.get('DEBUG_TIMER','').lower() in ['1','yes']:
-			env.log.timers = True
-		if os.environ.get('DEBUG_PARSER','').lower() in ['1','yes']:
-			env.log.parser = True
-		if os.environ.get('DEBUG_ROUTE','').lower() in ['1','yes']:
-			env.log.routes = True
-		if os.environ.get('DEBUG_ROUTES','').lower() in ['1','yes']:  # DEPRECATED even in 2.0.x
-			env.log.routes = True
-		if os.environ.get('DEBUG_ALL','').lower() in ['1','yes']:
-			env.log.all = True
-		if os.environ.get('DEBUG_CORE','').lower() in ['1','yes']:
-			env.log.reactor = True
-			env.log.daemon = True
-			env.log.processes = True
-			env.log.message = True
-			env.log.timer = True
-			env.log.routes = True
-			env.log.parser = False
-
-		pid = os.environ.get('PID','')
-		if pid:
-			env.daemon.pid = pid
-
-		try:
-			me = pwd.getpwuid(os.getuid()).pw_name
-			user = os.environ.get('USER','')
-			if user and user != 'root' and user != me and env.daemon.user == 'nobody':
-				env.daemon.user = user
-		except KeyError:
-			pass
-
-		daemon = os.environ.get('DAEMONIZE','').lower() in ['1','yes']
-		if daemon:
-			env.daemon.daemonize = True
-			env.log.enable = False
-
-		return env
-
 
 # ========================================================================= _env
 #
 
-import ConfigParser
+from exabgp.vendoring.six.moves import configparser as ConfigParser
 from exabgp.util.hashtable import HashTable
 
 
 def _env (conf):
-	here = os.path.join(os.sep,*os.path.join(environment.location.split(os.sep)))
+	here = os.path.join(os.sep,os.path.join(*environment.location.split(os.sep)))
 
 	location, directory = os.path.split(here)
 	while directory:
@@ -386,7 +340,10 @@ def _env (conf):
 				elif rep_name in os.environ:
 					conf = os.environ.get(rep_name)
 				else:
-					conf = environment.unquote(ini.get(proxy_section,option,nonedict))
+					if sys.version_info[0] < 3:
+						conf = environment.unquote(ini.get(proxy_section,option,nonedict))
+					else:
+						conf = environment.unquote(ini.get(proxy_section,option,vars=nonedict))
 					# name without an = or : in the configuration and no value
 					if conf is None:
 						conf = default[option]['value']
@@ -397,4 +354,4 @@ def _env (conf):
 			except TypeError:
 				raise environment.Error('invalid value for %s.%s : %s' % (section,option,conf))
 
-	return environment._compatibility(env)
+	return env
