@@ -2,9 +2,9 @@
 
 # based on netlink.py at ....
 # https://gforge.inria.fr/scm/viewvc.php/canso/trunk/tools/netlink.py?view=markup&revision=1360&root=mehani&pathrev=1360
-# http://www.linuxjournal.com/article/7356?page=0,1
+# https://www.linuxjournal.com/article/7356?page=0,1
 # http://smacked.org/docs/netlink.pdf
-# RFC 3549
+# RFC 3549 - https://tools.ietf.org/html/rfc3549
 
 import os
 import socket
@@ -13,6 +13,8 @@ from struct import unpack
 from struct import calcsize
 from collections import namedtuple
 
+from exabgp.util import concat_bytes_i
+
 try:
 	getattr(socket,'AF_NETLINK')
 except AttributeError:
@@ -202,13 +204,13 @@ class Attributes (object):
 			raw = pack(cls.Header.PACK,length,atype) + payload
 			pad = pad(length) - len(raw)
 			if pad:
-				raw += '\0'*pad
+				raw += b'\0'*pad
 			return raw
 
-		return ''.join([_encode(k,v) for (k,v) in attributes.items()])
+		return concat_bytes_i(_encode(k,v) for (k,v) in attributes.items())
 
 
-class _InfoMessage (object):
+class _Message (object):
 	# to be defined by the subclasses
 	format = namedtuple('Parent', 'to be subclassed')
 
@@ -243,7 +245,7 @@ class _InfoMessage (object):
 # |                      Change Mask                              |
 # +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
 
-class Link(_InfoMessage):
+class Link(_Message):
 	class Header (object):
 		PACK = 'BxHiII'
 		LEN = calcsize(PACK)
@@ -308,7 +310,7 @@ class Link(_InfoMessage):
 # |                     Interface Index                         |
 # +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
 
-class Address (_InfoMessage):
+class Address (_Message):
 	class Header (object):
 		PACK = '4Bi'
 		LEN = calcsize(PACK)
@@ -380,7 +382,7 @@ class Address (_InfoMessage):
 # |           State             |     Flags     |     Type      |
 # +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
 
-class Neighbor (_InfoMessage):
+class Neighbor (_Message):
 	class Header (object):
 		# linux/if_addr.h
 		PACK = 'BxxxiHBB'
@@ -437,7 +439,7 @@ class Neighbor (_InfoMessage):
 # |                          Flags                              |
 # +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
 
-class Network (_InfoMessage):
+class Network (_Message):
 	class Header (object):
 		# linux/if_addr.h
 		PACK = '8BI'  # or is it 8Bi ?
@@ -525,9 +527,10 @@ class Network (_InfoMessage):
 	def getRoutes (self):
 		return self.extract(Network.Command.RTM_GETROUTE)
 
-	def _create (self, family):
-		for _ in self.extract(Network.Command.RTM_NEWROUTE, flags,family):
-			yield _
+	# def _create (self, family):
+	# 	raise RuntimeError('need to fix this flag thing')
+	# 	for _ in self.extract(Network.Command.RTM_NEWROUTE,flags,family):
+	# 		yield _
 
 	def newRoute (self):
 		network_flags  = NetLinkRoute.Flags.NLM_F_REQUEST
@@ -538,7 +541,7 @@ class Network (_InfoMessage):
 		family = socket.AF_INET
 
 		attributes = Attributes()
-		Network.Attribute.RTA_DST
+		# Network.Type.Attribute.RTA_DST
 
 		neighbor = self.format(
 			family,
@@ -549,14 +552,12 @@ class Network (_InfoMessage):
 			Network.Type.Protocol.RTPROT_EXABGP,
 			Network.Type.Scope.RT_SCOPE_UNIVERSE,
 			0,     # type
-			Network.Flag.RTM_F_PREFIX,  # this may be wrong
+			Network.Type.Flag.RTM_F_PREFIX,  # this may be wrong
 			attributes
 		)
 
-
 		# prefix = '\20\x7f\0\0\2'
 
-
 		for _ in self.extract(Network.Command.RTM_NEWROUTE,network_flags,family):
 			yield _
 
@@ -578,7 +579,7 @@ class Network (_InfoMessage):
 # +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
 
 
-class TC (_InfoMessage):
+class TC (_Message):
 	class Header (object):
 		PACK = "BxxxiIII"
 		LEN = calcsize(PACK)
@@ -638,7 +639,7 @@ class TC (_InfoMessage):
 # +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
 
 
-class Firewall (_InfoMessage):
+class Firewall (_Message):
 	class Header (object):
 		PACK = "BxxxI"
 		LEN = calcsize(PACK)
