@@ -3,7 +3,8 @@
 usage.py
 
 Created by Thomas Mangin on 2009-09-06.
-Copyright (c) 2009-2015 Exa Networks. All rights reserved.
+Copyright (c) 2009-2017 Exa Networks. All rights reserved.
+License: 3-clause BSD. (See the COPYRIGHT file)
 """
 
 import sys
@@ -11,12 +12,12 @@ import resource
 
 if sys.platform == 'darwin':
 	# darwin returns bytes
-	divisor = 1024.0*1024.0
+	DIVISOR = 1024.0*1024.0
 else:
 	# other OS (AFAIK) return a number of pages
-	divisor = 1024.0*1024.0/resource.getpagesize()
+	DIVISOR = 1024.0*1024.0/resource.getpagesize()
 
 
 def usage (label='usage'):
 	rusage = resource.getrusage(resource.RUSAGE_SELF)
-	return '%s: usertime=%s systime=%s mem=%s mb' % (label,rusage.ru_utime,rusage.ru_stime,(rusage.ru_maxrss/divisor))
+	return '%s: usertime=%s systime=%s mem=%s mb' % (label,rusage.ru_utime,rusage.ru_stime,(rusage.ru_maxrss/DIVISOR))
