@@ -5,11 +5,43 @@ decode.py
 
 Created by Thomas Mangin on 2009-09-06.
 Copyright (c) 2009-2015 Exa Networks. All rights reserved.
+License: 3-clause BSD. (See the COPYRIGHT file)
 """
 
 # pylint: ignore=E131
 # pylama:ignore=E131
 
+import sys
+
+# INET include is required as it perform some AFI/SAFI registration
+from exabgp.bgp.message.update.nlri import INET  # noqa
+
+from exabgp.bgp.message.open.routerid import RouterID
+from exabgp.protocol.ip import IPv4
+from exabgp.bgp.message.open.asn import ASN
+from exabgp.bgp.message.open.holdtime import HoldTime
+
+from exabgp.bgp.message import Update
+from exabgp.bgp.message import Open
+from exabgp.bgp.message.open import Version
+from exabgp.bgp.message.open import ASN
+from exabgp.bgp.message.open import RouterID
+from exabgp.bgp.message.open import HoldTime
+from exabgp.bgp.message.open.capability import Capabilities
+from exabgp.bgp.message.open.capability import Capability
+from exabgp.bgp.message.open.capability import Negotiated
+from exabgp.bgp.message.update.nlri import NLRI
+
+
+from exabgp.configuration.setup import environment
+from exabgp.logger import Logger
+
+from exabgp.util.test import data_from_body
+
+environment.setup('')
+logger = Logger()
+
+
 bodies = []
 
 body = [
@@ -193,35 +225,87 @@ body = [
 	0x16, 0x1, 0x0, 0x1c,
 	0x17, 0x1, 0x0, 0x1a
 ]
+# asn4, data
+bodies.append((True,body))
+
+# BGP-LS with draft-gredler-idr-bgp-ls-segment-routing-ext
+body = [0x00, 0x00, # withdrawn routes length 0
+		0x00, 0xdc,	# Path Attribute Length 220
+		0x90, 0x0e,
+		0x00, 0x62,
+		0x40, 0x04, 0x47, 0x04,
+		0xac, 0x18, 0xf8, 0x4f,
+		0x00, 0x00, 0x02, 0x00,
+		0x55, 0x02, 0x00, 0x00,
+		0x00, 0x00, 0x00, 0x00,
+		0x00, 0x00, 0x01, 0x00,
+		0x00, 0x1a, 0x02, 0x00,
+		0x00, 0x04, 0x00, 0x00,
+		0xfd, 0xe8, 0x02, 0x01,
+		0x00, 0x04, 0xac, 0x10,
+		0x00, 0x58, 0x02, 0x03,
+		0x00, 0x06, 0x17, 0x20,
+		0x00, 0x16, 0x00, 0x02,
+		0x01, 0x01, 0x00, 0x1a,
+		0x02, 0x00, 0x00, 0x04,
+		0x00, 0x00, 0xfd, 0xe8,
+		0x02, 0x01, 0x00, 0x04,
+		0xac, 0x10, 0x00, 0x58,
+		0x02, 0x03, 0x00, 0x06,
+		0x17, 0x20, 0x00, 0x16,
+		0x00, 0x01, 0x01, 0x03,
+		0x00, 0x04, 0x0a, 0x00,
+		0x00, 0x07, 0x01, 0x04,
+		0x00, 0x04, 0x0a, 0x00,
+		0x00, 0x06, 0x40, 0x01,
+		0x01, 0x00, 0x40, 0x02,
+		0x06, 0x02, 0x01, 0x00,
+		0x00, 0xfd, 0xe8,
+		0x80, 0x1d, 			# Link State Attribute Code 29
+		0x66,  					# Length
+		0x04, 0x04, 0x00, 0x04, # Type 1028, Length 4
+		0xac, 0x10, 0x00, 0x02, # IPv4 Router-ID 172.16.0.2
+		0x04, 0x06,   			# Type 1030
+		0x00, 0x04,
+		0xac, 0x10, 0x00, 0x01, # IPv4 Router ID 172.16.0.1
+		0x04, 0x40,				# Type 1088
+		0x00, 0x04,
+		0x00, 0x00, 0x00, 0x00, # SRLG = 0
+		0x04, 0x41,				# Type 1089
+		0x00, 0x04,
+		0x4c, 0xee, 0x6b, 0x28, # 1000.00 Mbps Maximum Link bandwidth
+		0x04, 0x42,				# Type 1090
+		0x00, 0x04,
+		0x4c, 0xee, 0x6b, 0x28, # 1000.00 Mbps Maximum reservable link bandwidth
+		0x04, 0x43,				# Type 1091
+		0x00, 0x20,
+		0x4c, 0xee, 0x6b, 0x28, # 1000.00 Mbps Unreserved Bandwidth P0
+		0x4c, 0xee, 0x6b, 0x28, # 1000.00 Mbps Unreserved Bandwidth P1
+		0x4c, 0xee, 0x6b, 0x28, # 1000.00 Mbps Unreserved Bandwidth P2
+		0x4c, 0xee, 0x6b, 0x28, # 1000.00 Mbps Unreserved Bandwidth P3
+		0x4c, 0xee, 0x6b, 0x28, # 1000.00 Mbps Unreserved Bandwidth P4
+		0x4c, 0xee, 0x6b, 0x28, # 1000.00 Mbps Unreserved Bandwidth P5
+		0x4c, 0xee, 0x6b, 0x28, # 1000.00 Mbps Unreserved Bandwidth P6
+		0x4c, 0xee, 0x6b, 0x28, # 1000.00 Mbps Unreserved Bandwidth P7
+		0x04, 0x44,				# Type 1092
+		0x00, 0x04,				# Length 4
+		0x00, 0x00, 0x00, 0x0a, # 10
+		0x04, 0x47,				# Type 1095 Metric TLV
+		0x00, 0x03,				# Length 3
+		0x00, 0x00, 0x0a,		# 10
+		0x04, 0x4b,				# Type 1099 Adjacency Segment TLV
+		0x00, 0x07,				# Length 7
+		0x30,					# Flags  0011 0000
+		0x00,					# Weigth 0
+		0x00, 0x00,				# Reserved 0x0
+		0x04, 0x93, 0x1e]		# SID/Label 299806
+
 
 # asn4, data
 bodies.append((True,body))
 
-# Prefix include is required as it perform some AFI/SAFI registration
-from exabgp.bgp.message.update.nlri.prefix import Prefix  # noqa
 
-from exabgp.bgp.message.open.routerid import RouterID
-from exabgp.protocol.ip import IPv4
-from exabgp.bgp.message.open.asn import ASN
-from exabgp.bgp.message.open.holdtime import HoldTime
-
-from exabgp.protocol.family import known_families
-from exabgp.bgp.message.update import Update
-from exabgp.bgp.message.open import Open
-from exabgp.bgp.message.open.capability.capabilities import Capabilities
-from exabgp.bgp.message.open.capability.capability import Capability
-from exabgp.bgp.message.open.capability.negotiated import Negotiated
-# from exabgp.bgp.message.notification import Notify
-
-
-from exabgp.configuration.setup import environment
-env = environment.setup('')
-
-from exabgp.logger import Logger
-logger = Logger()
-
-
-class Neighbor:
+class FakeNeighbor (object):
 	description = 'a test neighbor'
 	router_id = RouterID('127.0.0.1')
 	local_address = IPv4('127.0.0.1')
@@ -233,6 +317,8 @@ class Neighbor:
 	hold_time = HoldTime(180)
 	asn4 = False
 	add_path = 0
+	extended_message = False
+	nexthop = None
 
 	# capability
 	route_refresh = False
@@ -244,7 +330,7 @@ class Neighbor:
 
 	@staticmethod
 	def families ():
-		return known_families()
+		return NLRI.known_families()
 
 import unittest
 
@@ -263,24 +349,26 @@ import unittest
 
 class TestUpdateDecoding (unittest.TestCase):
 	def setUp (self):
-		# env.log.all = True
 		self.negotiated = {}
 
 		for asn4 in (True,False):
-			neighbor = Neighbor()
+			neighbor = FakeNeighbor()
 			neighbor.asn4 = asn4
 
 			capa = Capabilities().new(neighbor,False)
 			capa[Capability.CODE.MULTIPROTOCOL] = neighbor.families()
 
 			# path = {}
-			# for f in known_families():
+			# for f in NLRI.known_families():
 			# 	if neighbor.add_path:
 			# 		path[f] = neighbor.add_path
 			# capa[Capability.CODE.ADD_PATH] = path
 
-			o1 = Open(4,neighbor.local_as,'127.0.0.1',capa,180)
-			o2 = Open(4,neighbor.peer_as,'127.0.0.1',capa,180)
+			routerid_1 = str(neighbor.router_id)
+			routerid_2 = '.'.join(str((int(_)+1) % 250) for _ in str(neighbor.router_id).split('.',-1))
+
+			o1 = Open(Version(4),ASN(neighbor.local_as),HoldTime(180),RouterID(routerid_1),capa)
+			o2 = Open(Version(4),ASN(neighbor.peer_as),HoldTime(180),RouterID(routerid_2),capa)
 
 			negotiated = Negotiated(neighbor)
 			negotiated.sent(o1)
@@ -296,13 +384,13 @@ class TestUpdateDecoding (unittest.TestCase):
 		for asn4,body in bodies:
 			if asn4:
 				continue
-			Update.unpack_message(''.join(chr(_) for _ in body),self.negotiated[asn4])
+			Update.unpack_message(data_from_body(body),self.negotiated[asn4])
 
 	def test_decoding_udpate_asn4 (self):
 		for asn4,body in bodies:
 			if not asn4:
 				continue
-			Update.unpack_message(''.join(chr(_) for _ in body),self.negotiated[asn4])
+			Update.unpack_message(data_from_body(body),self.negotiated[asn4])
 
 if __name__ == '__main__':
 	unittest.main()
